@@ -47,12 +47,13 @@ fn main() {
         .default_enum_style(bindgen::EnumVariation::Rust {
             non_exhaustive: true,
         })
+        .bitfield_enum("btree_iter_update_trigger_flags")
         .allowlist_function("bcachefs_usage")
         .allowlist_function("raid_init")
         .allowlist_function("cmd_.*")
         .allowlist_function(".*_cmds")
         .allowlist_function(".*bch2_.*")
-        .allowlist_function("bcache_fs_open")
+        .allowlist_function("bcache_fs_open_fallible")
         .allowlist_function("bcache_fs_close")
         .allowlist_function("bio_.*")
         .allowlist_function("derive_passphrase")
@@ -64,6 +65,7 @@ fn main() {
         .blocklist_function("bch2_prt_vprintf")
         .blocklist_type("rhash_lock_head")
         .blocklist_type("srcu_struct")
+        .blocklist_type("bch_ioctl_data_event")
         .allowlist_var("BCH_.*")
         .allowlist_var("KEY_SPEC_.*")
         .allowlist_var("Fix753_.*")
@@ -77,12 +79,14 @@ fn main() {
         .rustified_enum("fsck_err_opts")
         .allowlist_type("nonce")
         .no_debug("bch_replicas_padded")
+        .no_debug("jset")
         .newtype_enum("bch_kdf_types")
         .rustified_enum("bch_key_types")
         .opaque_type("gendisk")
         .opaque_type("gc_stripe")
         .opaque_type("open_bucket.*")
         .opaque_type("replicas_delta_list")
+        .allowlist_type("sb_names")
         .no_copy("btree_trans")
         .no_copy("printbuf")
         .no_partialeq("bkey")
