@@ -90,7 +90,7 @@ struct move_btree_args {
 
 static bool move_btree_pred(struct bch_fs *c, void *_arg,
 			    enum btree_id btree, struct bkey_s_c k,
-			    struct bch_io_opts *io_opts,
+			    struct bch_inode_opts *io_opts,
 			    struct data_update_opts *data_opts)
 {
 	struct move_btree_args *args = _arg;
@@ -165,23 +165,19 @@ static void check_gaps(struct bch_fs *c)
 
 static int get_nbuckets_used(struct bch_fs *c, u64 *nbuckets)
 {
-	struct btree_trans *trans = bch2_trans_get(c);
-	struct btree_iter iter;
-	bch2_trans_iter_init(trans, &iter, BTREE_ID_alloc, POS(0, U64_MAX), 0);
+	CLASS(btree_trans, trans)(c);
+	CLASS(btree_iter, iter)(trans, BTREE_ID_alloc, POS(0, U64_MAX), 0);
 	struct bkey_s_c k;
-	int ret = lockrestart_do(trans, bkey_err(k = bch2_btree_iter_peek_prev(trans, &iter)));
+	int ret = lockrestart_do(trans, bkey_err(k = bch2_btree_iter_peek_prev(&iter)));
 	if (!ret && k.k->type != KEY_TYPE_alloc_v4)
 		ret = -ENOENT;
 	if (ret) {
 		fprintf(stderr, "error looking up last alloc key: %s\n", bch2_err_str(ret));
-		goto err;
+		return ret;
 	}
 
 	*nbuckets = (k.k->p.offset + 1);
-err:
-	bch2_trans_iter_exit(trans, &iter);
-	bch2_trans_put(trans);
-	return ret;
+	return 0;
 
 }
 
@@ -508,6 +504,7 @@ static void image_create_usage(void)
 	     "      --superblock_size=size\n"
 	     "      --bucket_size=size\n"
 	     "      --fs_size=size          Expected size of device image will be used on, hint for bucket size\n"
+	     "      --version=version       Create filesystem with specified on disk format version instead of the latest\n"
 	     "  -f, --force\n"
 	     "  -q, --quiet                 Only print errors\n"
 	     "  -v, --verbose               Verbose filesystem initialization\n"
@@ -525,6 +522,7 @@ static int cmd_image_create(int argc, char *argv[])
 		{ "fs_label",		required_argument,	NULL, 'L' },
 		{ "uuid",		required_argument,	NULL, 'U' },
 		{ "superblock_size",	required_argument,	NULL, 'S' },
+		{ "version",		required_argument,	NULL, 'V' },
 		{ "force",		no_argument,		NULL, 'f' },
 		{ "quiet",		no_argument,		NULL, 'q' },
 		{ "verbose",		no_argument,		NULL, 'v' },
@@ -600,6 +598,9 @@ static int cmd_image_create(int argc, char *argv[])
 		case 'v':
 			verbosity++;
 			break;
+		case 'V':
+			opts.version = version_parse(optarg);
+			break;
 		case 'h':
 			image_create_usage();
 			exit(EXIT_SUCCESS);
@@ -613,8 +614,10 @@ static int cmd_image_create(int argc, char *argv[])
 	}
 	args_shift(optind);
 
-	if (argc != 1)
+	if (argc != 1) {
+		image_create_usage();
 		die("Please supply a filename for the new image");
+	}
 
 	dev_opts.path = argv[0];
 
@@ -665,7 +668,10 @@ static int image_update(const char *src_path, const char *dst_image,
 			goto err;
 		}
 
-		if (ftruncate(dev_opts.bdev->bd_fd, input_bytes)) {
+		u64 metadata_dev_size = max(input_bytes,
+				c->opts.btree_node_size * BCH_MIN_NR_NBUCKETS);
+
+		if (ftruncate(dev_opts.bdev->bd_fd, metadata_dev_size)) {
 			fprintf(stderr, "ftruncate error: %m");
 			goto err;
 		}
@@ -677,10 +683,13 @@ static int image_update(const char *src_path, const char *dst_image,
 		if (ret)
 			goto err;
 
-		ret = bch2_dev_add(c, dev_opts.path);
-		bch_err_msg(c, ret, "adding metadata device");
-		if (ret)
+		CLASS(printbuf, err)();
+		ret = bch2_dev_add(c, dev_opts.path, &err);
+		if (ret) {
+			bch_err(c, "error adding metadata device: %s\n%s",
+				bch2_err_str(ret), err.buf);
 			goto err;
+		}
 	}
 
 	set_data_allowed_for_image_update(c);
@@ -777,8 +786,10 @@ static int cmd_image_update(int argc, char *argv[])
 		}
 	args_shift(optind);
 
-	if (argc != 1)
+	if (argc != 1) {
+		image_update_usage();
 		die("Please supply a filename");
+	}
 
 	return image_update(source, argv[0],
 			    keep_alloc, verbosity);
