@@ -130,52 +130,82 @@ struct bchfs_handle bchu_fs_open_by_dev(const char *, int *);
 
 int bchu_dev_path_to_idx(struct bchfs_handle, const char *);
 
+static inline bool __errmsg_ioctl(int fd, int nr, void *arg,
+				  struct bch_ioctl_err_msg *errmsg,
+				  const char *nr_str)
+{
+	char err[8192];
+	errmsg->msg_ptr = (ulong) err;
+	errmsg->msg_len = sizeof(err);
+	int ret = ioctl(fd, nr, arg);
+	if (ret < 0 && errno != ENOTTY)
+		die("%s error: %s\n%s", nr_str, bch2_err_str(-errno), err);
+	return ret >= 0;
+}
+
+#define errmsg_ioctl(_fd, _nr, _arg)			\
+	__errmsg_ioctl(_fd, _nr, _arg, &(_arg)->err, #_nr)
+
 static inline void bchu_disk_add(struct bchfs_handle fs, const char *dev)
 {
-	struct bch_ioctl_disk i = { .dev = (unsigned long) dev, };
+	struct bch_ioctl_disk_v2 i = { .dev = (unsigned long) dev, };
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_ADD, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_ADD_v2, &i)) {
+		struct bch_ioctl_disk i = { .dev = (unsigned long) dev, };
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_ADD, &i);
+	}
 }
 
 static inline void bchu_disk_remove(struct bchfs_handle fs, unsigned dev_idx,
 				    unsigned flags)
 {
-	struct bch_ioctl_disk i = {
-		.flags	= flags|BCH_BY_INDEX,
-		.dev	= dev_idx,
-	};
+	struct bch_ioctl_disk_v2 i = { .flags = flags|BCH_BY_INDEX, .dev = dev_idx, };
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_REMOVE, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_REMOVE_v2, &i)) {
+		struct bch_ioctl_disk i = { .flags = flags|BCH_BY_INDEX, .dev = dev_idx, };
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_REMOVE, &i);
+	}
 }
 
 static inline void bchu_disk_online(struct bchfs_handle fs, char *dev)
 {
-	struct bch_ioctl_disk i = { .dev = (unsigned long) dev, };
+	struct bch_ioctl_disk_v2 i = { .dev = (unsigned long) dev, };
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_ONLINE, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_ONLINE_v2, &i)) {
+		struct bch_ioctl_disk i = { .dev = (unsigned long) dev, };
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_ONLINE, &i);
+	}
 }
 
 static inline void bchu_disk_offline(struct bchfs_handle fs, unsigned dev_idx,
 				     unsigned flags)
 {
-	struct bch_ioctl_disk i = {
-		.flags	= flags|BCH_BY_INDEX,
-		.dev	= dev_idx,
-	};
+	struct bch_ioctl_disk_v2 i = { .flags = flags|BCH_BY_INDEX, .dev = dev_idx, };
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_OFFLINE, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_OFFLINE_v2, &i)) {
+		struct bch_ioctl_disk i = { .flags = flags|BCH_BY_INDEX, .dev = dev_idx, };
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_OFFLINE, &i);
+	}
 }
 
 static inline void bchu_disk_set_state(struct bchfs_handle fs, unsigned dev,
 				       unsigned new_state, unsigned flags)
 {
-	struct bch_ioctl_disk_set_state i = {
+	struct bch_ioctl_disk_set_state_v2 i = {
 		.flags		= flags|BCH_BY_INDEX,
 		.new_state	= new_state,
 		.dev		= dev,
 	};
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_SET_STATE, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_SET_STATE_v2, &i)) {
+		struct bch_ioctl_disk_set_state i = {
+			.flags		= flags|BCH_BY_INDEX,
+			.new_state	= new_state,
+			.dev		= dev,
+		};
+
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_SET_STATE, &i);
+	}
 }
 
 static inline struct bch_ioctl_fs_usage *bchu_fs_usage(struct bchfs_handle fs)
@@ -288,36 +318,53 @@ static inline void bchu_disk_resize(struct bchfs_handle fs,
 				    unsigned idx,
 				    u64 nbuckets)
 {
-	struct bch_ioctl_disk_resize i = {
+	struct bch_ioctl_disk_resize_v2 i = {
 		.flags	= BCH_BY_INDEX,
 		.dev	= idx,
 		.nbuckets = nbuckets,
 	};
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_RESIZE, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_RESIZE_v2, &i)) {
+		struct bch_ioctl_disk_resize i = {
+			.flags	= BCH_BY_INDEX,
+			.dev	= idx,
+			.nbuckets = nbuckets,
+		};
+
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_RESIZE, &i);
+	}
 }
 
 static inline void bchu_disk_resize_journal(struct bchfs_handle fs,
 					    unsigned idx,
 					    u64 nbuckets)
 {
-	struct bch_ioctl_disk_resize i = {
+	struct bch_ioctl_disk_resize_v2 i = {
 		.flags	= BCH_BY_INDEX,
 		.dev	= idx,
 		.nbuckets = nbuckets,
 	};
 
-	xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_RESIZE_JOURNAL, &i);
+	if (!errmsg_ioctl(fs.ioctl_fd, BCH_IOCTL_DISK_RESIZE_JOURNAL_v2, &i)) {
+		struct bch_ioctl_disk_resize i = {
+			.flags	= BCH_BY_INDEX,
+			.dev	= idx,
+			.nbuckets = nbuckets,
+		};
+
+		xioctl(fs.ioctl_fd, BCH_IOCTL_DISK_RESIZE_JOURNAL, &i);
+	}
 }
 
 int bchu_data(struct bchfs_handle, struct bch_ioctl_data);
 
 struct dev_name {
-	unsigned	idx;
-	char		*dev;
-	char		*label;
-	uuid_t		uuid;
-	unsigned	durability;
+	unsigned		idx;
+	char			*dev;
+	char			*label;
+	uuid_t			uuid;
+	unsigned		durability;
+	enum bch_member_state	state;
 };
 typedef DARRAY(struct dev_name) dev_names;
 
