@@ -22,6 +22,7 @@
 #include "linux/sort.h"
 #include "tools-util.h"
 #include "libbcachefs/util.h"
+#include "src/rust_to_c.h"
 
 void die(const char *fmt, ...)
 {
@@ -51,15 +52,15 @@ char *mprintf(const char *fmt, ...)
 	return str;
 }
 
-void xpread(int fd, void *buf, size_t count, off_t offset)
+void __xpread(int fd, void *buf, size_t count, off_t offset, const char *file, unsigned line)
 {
 	while (count) {
 		ssize_t r = pread(fd, buf, count, offset);
 
 		if (r < 0)
-			die("read error: %m");
+			die("read error: %m at %s:%u", file, line);
 		if (!r)
-			die("pread error: unexpected eof");
+			die("pread error: unexpected eof at %s:%u", file, line);
 		count	-= r;
 		offset	+= r;
 	}
@@ -156,6 +157,16 @@ ssize_t read_string_list_or_die(const char *opt, const char * const list[],
 	return v;
 }
 
+u64 read_flag_list_or_die(char *opt, const char * const list[],
+			  const char *msg)
+{
+	u64 v = bch2_read_flag_list(opt, list);
+	if (v == (u64) -1)
+		die("Bad %s %s", msg, opt);
+
+	return v;
+}
+
 /* Returns size of file or block device: */
 u64 get_size(int fd)
 {
@@ -183,7 +194,7 @@ unsigned get_blocksize(int fd)
 }
 
 /* Open a block device, do magic blkid stuff to probe for existing filesystems: */
-int open_for_format(struct dev_opts *dev, bool force)
+int open_for_format(struct dev_opts *dev, blk_mode_t mode, bool force)
 {
 	int blkid_version_code = blkid_get_library_version(NULL, NULL);
 	if (blkid_version_code < 2401) {
@@ -208,7 +219,7 @@ int open_for_format(struct dev_opts *dev, bool force)
 	size_t fs_type_len, fs_label_len;
 
 	dev->file = bdev_file_open_by_path(dev->path,
-				BLK_OPEN_READ|BLK_OPEN_WRITE|BLK_OPEN_EXCL|BLK_OPEN_BUFFERED,
+				BLK_OPEN_READ|BLK_OPEN_WRITE|BLK_OPEN_EXCL|BLK_OPEN_BUFFERED|mode,
 				dev, NULL);
 	int ret = PTR_ERR_OR_ZERO(dev->file);
 	if (ret < 0)
@@ -281,11 +292,16 @@ static int range_cmp(const void *_l, const void *_r)
 	return 0;
 }
 
+void ranges_sort(ranges *r)
+{
+	sort(r->data, r->nr, sizeof(r->data[0]), range_cmp, NULL);
+}
+
 void ranges_sort_merge(ranges *r)
 {
 	ranges tmp = { 0 };
 
-	sort(r->data, r->nr, sizeof(r->data[0]), range_cmp, NULL);
+	ranges_sort(r);
 
 	/* Merge contiguous ranges: */
 	darray_for_each(*r, i) {
@@ -547,7 +563,7 @@ char *dev_to_path(dev_t dev)
 	return path;
 }
 
-struct mntent *dev_to_mount(char *dev)
+struct mntent *dev_to_mount(const char *dev)
 {
 	struct mntent *mnt, *ret = NULL;
 	FILE *f = setmntent("/proc/mounts", "r");
@@ -586,7 +602,7 @@ found:
 	return ret;
 }
 
-int dev_mounted(char *dev)
+int dev_mounted(const char *dev)
 {
 	struct mntent *mnt = dev_to_mount(dev);
 
@@ -671,6 +687,15 @@ static int kstrtouint_symbolic(const char *s, unsigned int base, unsigned *res)
 
 struct bpos bpos_parse(char *buf)
 {
+	if (!strcmp(buf, "POS_MIN"))
+		return POS_MIN;
+
+	if (!strcmp(buf, "POS_MAX"))
+		return POS_MAX;
+
+	if (!strcmp(buf, "SPOS_MAX"))
+		return SPOS_MAX;
+
 	char *orig = strdup(buf);
 	char *s = buf;
 
@@ -748,12 +773,14 @@ unsigned version_parse(char *buf)
 	return BCH_VERSION(major, minor);
 }
 
-darray_str get_or_split_cmdline_devs(int argc, char *argv[])
+darray_const_str get_or_split_cmdline_devs(int argc, char *argv[])
 {
-	darray_str ret = {};
+	darray_const_str ret = {};
 
 	if (argc == 1) {
-		bch2_split_devs(argv[0], &ret);
+		char *dev = bch2_scan_devices(argv[0]);
+
+		bch2_split_devs(dev, &ret);
 	} else {
 		for (unsigned i = 0; i < argc; i++)
 			darray_push(&ret, strdup(argv[i]));
