@@ -141,14 +141,16 @@ void bch2_io_error_work(struct work_struct *work)
 		if (ca->mi.state >= BCH_MEMBER_STATE_ro)
 			return;
 
-		bool dev = !__bch2_dev_set_state(c, ca, BCH_MEMBER_STATE_ro,
-						 BCH_FORCE_IF_DEGRADED);
 		CLASS(printbuf, buf)();
 		__bch2_log_msg_start(ca->name, &buf);
 
-		prt_printf(&buf, "writes erroring for %u seconds, setting %s ro",
-			c->opts.write_error_timeout,
-			dev ? "device" : "filesystem");
+		prt_printf(&buf, "writes erroring for %u seconds\n",
+			c->opts.write_error_timeout);
+
+		bool dev = !__bch2_dev_set_state(c, ca, BCH_MEMBER_STATE_ro,
+						 BCH_FORCE_IF_DEGRADED, &buf);
+
+		prt_printf(&buf, "setting %s ro", dev ? "device" : "filesystem");
 		if (!dev)
 			bch2_fs_emergency_read_only2(c, &buf);
 
@@ -392,7 +394,7 @@ int bch2_fsck_err_opt(struct bch_fs *c,
 		flags |= fsck_flags_extra[err];
 
 	if (test_bit(BCH_FS_in_fsck, &c->flags) ||
-	    test_bit(BCH_FS_in_recovery, &c->flags)) {
+	    c->opts.fix_errors != FSCK_FIX_exit) {
 		if (!(flags & (FSCK_CAN_FIX|FSCK_CAN_IGNORE)))
 			return bch_err_throw(c, fsck_repair_unimplemented);
 
@@ -466,10 +468,10 @@ int __bch2_fsck_err(struct bch_fs *c,
 
 	if ((flags & FSCK_ERR_SILENT) ||
 	    test_bit(err, c->sb.errors_silent)) {
-		ret = flags & FSCK_CAN_FIX
+		set_bit(BCH_FS_errors_fixed_silent, &c->flags);
+		return flags & FSCK_CAN_FIX
 			? bch_err_throw(c, fsck_fix)
 			: bch_err_throw(c, fsck_ignore);
-		goto err;
 	}
 
 	printbuf_indent_add_nextline(out, 2);
